Imports OmicronLab.TicroControl.TicroService
Imports System.Collections.Generic
Imports System.Configuration
Imports System.Data
Imports System.Linq
Imports System.ServiceModel
Imports System.Threading.Tasks
Imports System.Windows
Imports OmicronLab.TicroControl.Properties

''' <summary>
''' Interaction logic for App.xaml
''' </summary>
Partial Public Class App
	Inherits Application
	''' <summary>Gets or sets the API client.</summary>
	''' <value>The API client.</value>
	Public Shared Property ApiClient() As ApiClient
		Get
			Return m_ApiClient
		End Get
		Set(value As ApiClient)
			m_ApiClient = value
		End Set
	End Property
	Private Shared m_ApiClient As ApiClient

	''' <summary>Gets the API client for the supplied endpoint.</summary>
	''' <param name="endpoint">The endpoint.</param>
	''' <returns></returns>
	Friend Shared Function GetApiClient(endpoint As String) As ApiClient
		'The "AllowCookies = true" is required to keep the session when login is enabled
		Return New ApiClient(If(endpoint.ToLower().StartsWith("https"), DirectCast(New BasicHttpsBinding() With { _
			.AllowCookies = True _
		}, HttpBindingBase), New BasicHttpBinding() With { _
			.AllowCookies = True _
		}), New EndpointAddress(endpoint))
	End Function

	''' <summary>Connects to the service asynchronously.</summary>
	''' <returns></returns>
	Public Shared Async Function ConnectAsync() As Task(Of Boolean)
		Try
			System.Net.ServicePointManager.Expect100Continue = False
			'TICRO web server does not like the headers otherwise
			Dim endpoint As String = Settings.[Default].SoapEndpoint
			Dim client As ApiClient = Await Task(Of ApiClient).Run(Function() GetApiClient(endpoint))
			Dim status As StatusResponse = Await client.StatusAsync()
			If status.Body.StatusResult = "OK" Then
				ApiClient = client
			Else
				Return False
			End If
		Catch exp As AggregateException
			If TypeOf exp.InnerException Is FaultException Then
				Return HandleFault(TryCast(exp.InnerException, FaultException))
			Else
				Return HandleConnectException()
			End If
		Catch
			Return HandleConnectException()
		End Try
		Return True
	End Function
	''' <summary>Connects to the service.</summary>
	''' <returns></returns>
	Public Shared Function Connect(Optional password As String = Nothing) As Boolean
		Try
			System.Net.ServicePointManager.Expect100Continue = False
			'TICRO web server does not like the headers otherwise
			Dim endpoint As String = Settings.[Default].SoapEndpoint
			Dim client As ApiClient = GetApiClient(endpoint)
			If Not [String].IsNullOrWhiteSpace(password) Then
				If Not client.Login(password) Then
					Return False
				End If
			End If
			Dim status As String = client.Status()
			If status = "OK" Then
				ApiClient = client
			Else
				Return False
			End If
		Catch fault As FaultException
			Return If([String].IsNullOrWhiteSpace(password), HandleFault(fault), False)
		Catch
			Return HandleConnectException()
		End Try

		Return True
	End Function

	Private Shared Function HandleFault(fault As FaultException) As Boolean
		'If login is enabled an "Authentication" fault is returned
		If fault.Code.Name = "Authentication" Then
			Return If([String].IsNullOrWhiteSpace(Settings.[Default].Password), False, Connect(Settings.[Default].Password))
		Else
			Return HandleConnectException()
		End If
	End Function

	Private Shared Function HandleConnectException() As Boolean
		ApiClient = Nothing
		Return False
	End Function
End Class
