Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows.Input
Imports Microsoft.Practices.Prism.Commands
Imports OmicronLab.TicroControl.Interfaces

''' <summary>
''' See http://jake.ginnivan.net/awaitable-delegatecommand
''' </summary>
Public Class AwaitableDelegateCommand
	Inherits AwaitableDelegateCommand(Of Object)
	Implements IAsyncCommand
	Public Sub New(executeMethod As Func(Of Task))
		MyBase.New(Function(o) executeMethod())
	End Sub

	Public Sub New(executeMethod As Func(Of Task), canExecuteMethod As Func(Of Boolean))
		MyBase.New(Function(o) executeMethod(), Function(o) canExecuteMethod())
	End Sub
End Class

Public Class AwaitableDelegateCommand(Of T As Class)
	Implements IAsyncCommand(Of T)
	Implements ICommand
	Private ReadOnly _executeMethod As Func(Of T, Task)
	Private ReadOnly _underlyingCommand As DelegateCommand(Of T)
	Private _isExecuting As Boolean

	Public Sub New(executeMethod As Func(Of T, Task))
		Me.New(executeMethod, Function() True)
	End Sub

	Public Sub New(executeMethod As Func(Of T, Task), canExecuteMethod As Func(Of T, Boolean))
		_executeMethod = executeMethod
		_underlyingCommand = New DelegateCommand(Of T)(Sub(x)
															 End Sub, canExecuteMethod)
	End Sub

	Public Async Function ExecuteAsync(obj As T) As Task Implements IAsyncCommand(Of T).ExecuteAsync
		Try
			_isExecuting = True
			RaiseCanExecuteChanged()
			Await _executeMethod(obj)
		Finally
			_isExecuting = False
			RaiseCanExecuteChanged()
		End Try
	End Function

	Public ReadOnly Property Command() As ICommand Implements IAsyncCommand(Of T).Command
		Get
			Return Me
		End Get
	End Property

	Public Function CanExecute(parameter As Object) As Boolean Implements IAsyncCommand(Of T).CanExecute, ICommand.CanExecute
		Return Not _isExecuting AndAlso _underlyingCommand.CanExecute(DirectCast(parameter, T))
	End Function

	Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
		AddHandler(ByVal value As EventHandler)
			AddHandler _underlyingCommand.CanExecuteChanged, value
		End AddHandler
		RemoveHandler(ByVal value As EventHandler)
			RemoveHandler _underlyingCommand.CanExecuteChanged, value
		End RemoveHandler
		RaiseEvent(ByVal sender As System.Object, ByVal e As System.EventArgs)
		End RaiseEvent
	End Event

	Public Async Sub Execute(parameter As Object) Implements ICommand.Execute
		Await ExecuteAsync(TryCast(parameter, T))
	End Sub

	Public Sub RaiseCanExecuteChanged() Implements IRaiseCanExecuteChanged.RaiseCanExecuteChanged
		_underlyingCommand.RaiseCanExecuteChanged()
	End Sub
End Class
