Imports FirstFloor.ModernUI.Windows
Imports Newtonsoft.Json.Linq
Imports OmicronLab.TicroControl.Helper
Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports OmicronLab.TicroControl.TicroService

Namespace OmicronLab.TicroControl.Content.Live
	''' <summary>Interaction logic for General.xaml</summary>
	Partial Public Class General
		Inherits UserControl
		Implements IContent
		Private isUpdating As Boolean = False
		Private updateTimer As System.Timers.Timer = Nothing

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			If updateTimer Is Nothing Then
				updateTimer = New System.Timers.Timer(500)
				updateTimer.AutoReset = True
				AddHandler updateTimer.Elapsed, Sub(s, arg)
													Try
														If Not isUpdating Then
															isUpdating = True
															UpdateValuesAsync().Wait()
															isUpdating = False
														End If
													Catch exp As AggregateException
														If Not (TypeOf exp.InnerException Is TaskCanceledException) Then
															Throw
														End If
													Catch generatedExceptionName As TaskCanceledException
													End Try
												End Sub
			End If
			updateTimer.Enabled = True
			Task.Run(Sub() UpdateValuesAsync().Wait())
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
			If updateTimer IsNot Nothing Then
				updateTimer.Enabled = False
			End If
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			'Download the JSON from the device and parse it - the example uses JSON.net
			Dim device As JObject = Await DownloadHelper.DownloadJsonAsync("deviceStatus.php")
			'deviceStatus contains the output states and clock status
			Dim general As JObject = Await DownloadHelper.DownloadJsonAsync("generalStatus.php")
			'generalStatus contains the current time, uptime and some generic information
			Dim ptp As JObject = Await DownloadHelper.DownloadJsonAsync("ptpStatus.php")
			'ptpStatus contains the current state of the PTP daemon
			Dispatcher.Invoke(Sub()
								  'The values can then be accessed as strings as followed
								  labelDeviceStatus.Content = device("device")("status")
								  labelPtp.Content = ptp("port")(0)("portState")
								  labelTime.Content = general("general")("dateTime")
								  labelProductName.Content = general("general")("productName")
								  labelSerialNumber.Content = general("general")("serialNumber")
								  labelSoftwareVersion.Content = general("general")("softwareVersion")
								  labelKernelVersion.Content = general("general")("kernelVersion")
								  labelHardwareRevision.Content = general("general")("hardwareRevision")
								  labelUptime.Content = general("general")("uptime")
							  End Sub)

			'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
			'Alternatively you can use the following API methods, but they can not be called every second, as the device can not handle that many API requests
			'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
			'Dim currentTime As GetCurrentTimeResponse = Await App.ApiClient.GetCurrentTimeAsync()
			'Dim deviceStatus As GetDeviceStatusResponse = Await App.ApiClient.GetDeviceStatusAsync()
			'Dim deviceInfo As GetDeviceInformationResponse = Await App.ApiClient.GetDeviceInformationAsync()
			'Dim ptpStatus As GetPtpStatusResponse = Await App.ApiClient.GetPtpStatusAsync()
			'
			'Dispatcher.Invoke(Sub()
			'					  labelDeviceStatus.Content = deviceStatus.Body.GetDeviceStatusResult.ClockStatus
			'					  labelPtp.Content = ptpStatus.Body.GetPtpStatusResult.PortState
			'					  labelTime.Content = currentTime.Body.GetCurrentTimeResult
			'					  labelProductName.Content = deviceInfo.Body.GetDeviceInformationResult.Product
			'					  labelSerialNumber.Content = deviceInfo.Body.GetDeviceInformationResult.SerialNumber
			'					  labelSoftwareVersion.Content = deviceInfo.Body.GetDeviceInformationResult.SoftwareVersion
			'					  labelKernelVersion.Content = deviceInfo.Body.GetDeviceInformationResult.KernelVersion
			'					  labelHardwareRevision.Content = deviceInfo.Body.GetDeviceInformationResult.HardwareRevision
			'					  labelUptime.Content = deviceInfo.Body.GetDeviceInformationResult.Uptime
			'				  End Sub)
			'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
		End Function
	End Class
End Namespace
