Imports FirstFloor.ModernUI.Windows
Imports Newtonsoft.Json.Linq
Imports OmicronLab.TicroControl.Helper
Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports OmicronLab.TicroControl.TicroService

Namespace OmicronLab.TicroControl.Content.Live
	''' <summary>
	''' Interaction logic for Network.xaml
	''' </summary>
	Partial Public Class Network
		Inherits UserControl
		Implements IContent
		Private isUpdating As Boolean = False
		Private updateTimer As System.Timers.Timer = Nothing

		''' <summary>Initializes a new instance of the <see cref="Network"/> class.</summary>
		Public Sub New()
			InitializeComponent()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			If updateTimer Is Nothing Then
				updateTimer = New System.Timers.Timer(500)
				updateTimer.AutoReset = True
				AddHandler updateTimer.Elapsed, Sub(s, arg)
													Try
														If Not isUpdating Then
															isUpdating = True
															UpdateValuesAsync().Wait()
															isUpdating = False
														End If
													Catch exp As AggregateException
														If Not (TypeOf exp.InnerException Is TaskCanceledException) Then
															Throw
														End If
													Catch generatedExceptionName As TaskCanceledException
													End Try
												End Sub
			End If
			updateTimer.Enabled = True
			Task.Run(Sub() UpdateValuesAsync().Wait())
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
			If updateTimer IsNot Nothing Then
				updateTimer.Enabled = False
			End If
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			'Download the JSON from the device and parse it - the example uses JSON.net
			Dim network As JObject = Await DownloadHelper.DownloadJsonAsync("networkStatus.php")
			'networkStatus contains the current network settings
			Dispatcher.Invoke(Sub()
								  'The values can then be accessed as strings as followed
								  labelIPv4Address.Content = network("network")("IPv4Address")
								  labelIPv6Address.Content = network("network")("IPv6Address")
								  labelMacAddress.Content = network("network")("macAddress")
								  labelHostName.Content = network("network")("hostname")
								  labelDomainName.Content = network("network")("domainName")
							  End Sub)

			'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
			'Alternatively you can use the following API methods, but they can not be called every second, as the device can not handle that many API requests
			'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
			'Dim networkInfo As GetNetworkStatusResponse = Await App.ApiClient.GetNetworkStatusAsync()
			'
			'Dispatcher.Invoke(Sub()
			'					  labelIPv4Address.Content = networkInfo.Body.GetNetworkStatusResult.IPv4Address
			'					  labelIPv6Address.Content = networkInfo.Body.GetNetworkStatusResult.IPv6Address
			'					  labelMacAddress.Content = networkInfo.Body.GetNetworkStatusResult.MacAddress
			'					  labelHostName.Content = networkInfo.Body.GetNetworkStatusResult.Hostname
			'					  labelDomainName.Content = networkInfo.Body.GetNetworkStatusResult.DomainName
			'				  End Sub)
			'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
		End Function
	End Class
End Namespace
