Imports FirstFloor.ModernUI.Windows
Imports Newtonsoft.Json.Linq
Imports OmicronLab.TicroControl.Helper
Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes

Namespace OmicronLab.TicroControl.Content.Live
	''' <summary>
	''' Interaction logic for Output.xaml
	''' </summary>
	Partial Public Class Output
		Inherits UserControl
		Implements IContent
		Private isUpdating As Boolean = False
		Private updateTimer As System.Timers.Timer = Nothing
		Private refreshTimer As System.Timers.Timer = Nothing

		''' <summary>Initializes a new instance of the <see cref="Output"/> class.</summary>
		Public Sub New()
			InitializeComponent()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			If updateTimer Is Nothing Then
				updateTimer = New System.Timers.Timer(500)
				updateTimer.AutoReset = True
				AddHandler updateTimer.Elapsed, Sub(s, arg)
													Try
														If Not isUpdating Then
															isUpdating = True
															UpdateValuesAsync().Wait()
															isUpdating = False
														End If
													Catch exp As AggregateException
														If Not (TypeOf exp.InnerException Is TaskCanceledException) Then
															Throw
														End If
													Catch generatedExceptionName As TaskCanceledException
													End Try
												End Sub
			End If
			updateTimer.Enabled = True
			Task.Run(Sub() UpdateValuesAsync().Wait())

			If refreshTimer Is Nothing Then
				refreshTimer = New System.Timers.Timer(15000)
				refreshTimer.AutoReset = True
				AddHandler refreshTimer.Elapsed, Sub(s, arg)
													 Try
														 RefreshValuesAsync().Wait()
													 Catch exp As AggregateException
														 If Not (TypeOf exp.InnerException Is TaskCanceledException) Then
															 Throw
														 End If
													 Catch generatedExceptionName As TaskCanceledException
													 End Try
												 End Sub
			End If
			refreshTimer.Enabled = True
			Task.Run(Sub() RefreshValuesAsync().Wait())
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
			If updateTimer IsNot Nothing Then
				updateTimer.Enabled = False
			End If
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			'Download the JSON from the device and parse it - the example uses JSON.net
			Dim device As JObject = Await DownloadHelper.DownloadJsonAsync("deviceStatus.php")
			'deviceStatus contains the output states and clock status
			Dispatcher.Invoke(Sub()
								  'The values can then be accessed as strings as followed
								  labelDeviceStatus.Content = device("device")("status")
								  labelOcxoTuningValue.Content = device("device")("tuningValue")

								  'The values can then be accessed as booleans as followed
								  imageActiveOutput1.Visibility = If(device("outputs")(0).Value(Of Boolean)("active"), Visibility.Visible, Visibility.Collapsed)
								  imageInactiveOutput1.Visibility = If(device("outputs")(0).Value(Of Boolean)("active"), Visibility.Collapsed, Visibility.Visible)

								  imageActiveOutput2.Visibility = If(device("outputs")(1).Value(Of Boolean)("active"), Visibility.Visible, Visibility.Collapsed)
								  imageInactiveOutput2.Visibility = If(device("outputs")(1).Value(Of Boolean)("active"), Visibility.Collapsed, Visibility.Visible)

								  imageActiveOutput3.Visibility = If(device("outputs")(2).Value(Of Boolean)("active"), Visibility.Visible, Visibility.Collapsed)
								  imageInactiveOutput3.Visibility = If(device("outputs")(2).Value(Of Boolean)("active"), Visibility.Collapsed, Visibility.Visible)

								  imageActiveOutput4.Visibility = If(device("outputs")(3).Value(Of Boolean)("active"), Visibility.Visible, Visibility.Collapsed)
								  imageInactiveOutput4.Visibility = If(device("outputs")(3).Value(Of Boolean)("active"), Visibility.Collapsed, Visibility.Visible)

								  imageActiveOutput5.Visibility = If(device("outputs")(4).Value(Of Boolean)("active"), Visibility.Visible, Visibility.Collapsed)
								  imageInactiveOutput5.Visibility = If(device("outputs")(4).Value(Of Boolean)("active"), Visibility.Collapsed, Visibility.Visible)
							  End Sub)
		End Function

		''' <summary>Refreshes the values asynchronous.</summary>
		''' <returns></returns>
		Private Async Function RefreshValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			'Query the soap interface for the output status - "Dim outputsStatus = App.ApiClient.GetOuputStatus()" executes the same synchronously
			Dim outputsStatus As TicroService.GetOuputStatusResponse = Await App.ApiClient.GetOuputStatusAsync()

			'Query the soap interface for the general settings - "Dim generalSettings = App.ApiClient.GetGeneralSettings()" executes the same synchronously
			Dim generalSettings As TicroService.GetGeneralSettingsResponse = Await App.ApiClient.GetGeneralSettingsAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the GeneralConfiguration object can be accessed at "generalSettings.Body.GetGeneralSettingsResult"
								  'when queried synchronous a GeneralConfiguration object is returned directly
								  labelOutputsActive.Content = generalSettings.Body.GetGeneralSettingsResult.OutputsActive
								  labelCurrentTimezone.Content = generalSettings.Body.GetGeneralSettingsResult.Timezone

								  'if queried asynchronous the array with the configuration values can be accessed at "outputsStatus.Body.GetOuputStatusResult"
								  'when queried synchronous the array is returned directly
								  labelStatusOutput1.Content = outputsStatus.Body.GetOuputStatusResult(0).FriendlyName
								  labelStatusOutput1.Foreground = If(outputsStatus.Body.GetOuputStatusResult(0).Mute, Brushes.Orange, Brushes.Black)

								  labelStatusOutput2.Content = outputsStatus.Body.GetOuputStatusResult(1).FriendlyName
								  labelStatusOutput2.Foreground = If(outputsStatus.Body.GetOuputStatusResult(1).Mute, Brushes.Orange, Brushes.Black)

								  labelStatusOutput3.Content = outputsStatus.Body.GetOuputStatusResult(2).FriendlyName
								  labelStatusOutput3.Foreground = If(outputsStatus.Body.GetOuputStatusResult(2).Mute, Brushes.Orange, Brushes.Black)

								  labelStatusOutput4.Content = outputsStatus.Body.GetOuputStatusResult(3).FriendlyName
								  labelStatusOutput4.Foreground = If(outputsStatus.Body.GetOuputStatusResult(3).Mute, Brushes.Orange, Brushes.Black)

								  labelStatusOutput5.Content = outputsStatus.Body.GetOuputStatusResult(4).FriendlyName
								  labelStatusOutput5.Foreground = If(outputsStatus.Body.GetOuputStatusResult(4).Mute, Brushes.Orange, Brushes.Black)
							  End Sub)
		End Function
	End Class
End Namespace
