Imports FirstFloor.ModernUI.Windows
Imports OmicronLab.TicroControl.Helper
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports OmicronLab.TicroControl.TicroService

Namespace OmicronLab.TicroControl.Content.OutputStatus
	''' <summary>Interaction logic for Output.xaml</summary>
	Partial Public Class Output
		Inherits UserControl
		Implements IContent
		Private output As Integer

		''' <summary>Gets the refresh command.</summary>
		''' <value>The refresh command.</value>
		Public Property RefreshCommand() As AwaitableDelegateCommand
			Get
				Return m_RefreshCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_RefreshCommand = value
			End Set
		End Property
		Private m_RefreshCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="Output"/> class.</summary>
		Public Sub New()
			RefreshCommand = New AwaitableDelegateCommand(AddressOf RefreshValuesAsync)
			InitializeComponent()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
			output = Convert.ToInt32(e.Fragment)
			Task.Run(Sub() RefreshCommand.Execute(Nothing))
			labelHeader.Text = Properties.Resources.OutputStatus & " - Output " & output
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
		End Sub

		''' <summary>Refreshes the values asynchronous.</summary>
		''' <returns></returns>
		Private Async Function RefreshValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			'Query the soap interface for the output status - "Dim outputsStatus = App.ApiClient.GetOuputStatus()" executes the same synchronously
			Dim outputsStatus As GetOuputStatusResponse = Await App.ApiClient.GetOuputStatusAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the array with the configuration values can be accessed at "outputsStatus.Body.GetOuputStatusResult"
								  'when queried synchronous the array is returned directly
								  Dim config As OutputConfiguration = outputsStatus.Body.GetOuputStatusResult.First(Function(c) c.Id = output)

								  labelTimeBase.Content = If((config.Mode And CInt(OutputMode.Utc)) = CInt(OutputMode.Utc), "UTC", (If((config.Mode And CInt(OutputMode.LocalTime)) = CInt(OutputMode.LocalTime), "Local time", "TAI")))

								  labelMode.Content = config.FriendlyName
								  'or
								  '(config.Mode & (int)OutputMode.IrigB) == (int)OutputMode.IrigB ? "IRIG-B" :
								  '((config.Mode & (int)OutputMode.Dcf77) == (int)OutputMode.Dcf77 ? "DCF77" :
								  '((config.Mode & (int)OutputMode.Ppx) == (int)OutputMode.Ppx ? "PPX" : "Off"));
								  checkBoxMuted.IsChecked = config.Mute

								  'OutputMode is an enum helping to split up the mode bit-mask
								  checkBoxModulation.IsChecked = (config.Mode And CInt(OutputMode.Modulated)) = CInt(OutputMode.Modulated)
								  'IrigExpressions is an enum helping to split up the IRIG-B type bit-mask
								  checkBoxControlFunctions.IsChecked = (config.IrigbConfiguration.Type And CInt(IrigExpressions.ControlFunctions)) = CInt(IrigExpressions.ControlFunctions)
								  checkBoxSBS.IsChecked = (config.IrigbConfiguration.Type And CInt(IrigExpressions.StraightBinarySeconds)) = CInt(IrigExpressions.StraightBinarySeconds)
								  checkBoxBcdYear.IsChecked = (config.IrigbConfiguration.Type And CInt(IrigExpressions.BcdYear)) = CInt(IrigExpressions.BcdYear)

								  'PpxMode is an enum helping to detect the PPX mode
								  labelPpx.Content = CType(config.PpxConfiguration.Pps, PpxMode).ToString()
								  labelPeriod.Content = config.PpxConfiguration.Ppm & " s"
								  labelPulseWidth.Content = config.PpxConfiguration.PulseWidth & " ns"
								  'TimeReference is an enum helping to detect the PPX reference
								  labelTimeReference.Content = CType(config.PpxConfiguration.PulseSlope, TimeReference).ToString()

								  labelTrigger.Content = config.TriggerTime
							  End Sub)
		End Function
	End Class
End Namespace
