Imports FirstFloor.ModernUI.Windows
Imports OmicronLab.TicroControl.Helper
Imports OmicronLab.TicroControl.TicroService
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes

Namespace OmicronLab.TicroControl.Content.Settings
	''' <summary>Interaction logic for Output.xaml</summary>
	Partial Public Class Output
		Inherits UserControl
		Implements IContent
		Private output As Integer

		''' <summary>Gets the save command.</summary>
		''' <value>The save command.</value>
		Public Property SaveCommand() As AwaitableDelegateCommand
			Get
				Return m_SaveCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_SaveCommand = Value
			End Set
		End Property
		Private m_SaveCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="General"/> class.</summary>
		Public Sub New()
			SaveCommand = New AwaitableDelegateCommand(AddressOf SaveAsync)

			InitializeComponent()

			comboBoxTimeBase.ItemsSource = [Enum].GetValues(GetType(TimeBase)).Cast(Of TimeBase)()
			comboBoxMode.ItemsSource = New List(Of OutputMode)() From { _
				OutputMode.Off, _
				OutputMode.IrigB, _
				OutputMode.Ppx, _
				OutputMode.Dcf77 _
			}
			comboBoxPpx.ItemsSource = [Enum].GetValues(GetType(PpxMode)).Cast(Of PpxMode)()
			comboBoxTimeReference.ItemsSource = [Enum].GetValues(GetType(TimeReference)).Cast(Of TimeReference)()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
			output = Convert.ToInt32(e.Fragment)
			Task.Run(Sub() UpdateValuesAsync().Wait())
			labelHeader.Text = Properties.Resources.OutputStatus & " - Output " & output
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Dispatcher.Invoke(Function() Xceed.Wpf.Toolkit.MessageBox.Show("Please connect to a device to load the current values."))
				Return
			End If

			'Query the soap interface for the output status - "Dim outputsStatus = App.ApiClient.GetOuputStatus()" executes the same synchronously
			Dim outputsStatus As GetOuputStatusResponse = Await App.ApiClient.GetOuputStatusAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the array with the configuration values can be accessed at "outputsStatus.Body.GetOuputStatusResult"
								  'when queried synchronous the array is returned directly
								  Dim config As OutputConfiguration = outputsStatus.Body.GetOuputStatusResult.First(Function(c) c.Id = output)

								  comboBoxTimeBase.SelectedItem = If((config.Mode And CInt(OutputMode.Utc)) = CInt(OutputMode.Utc), TimeBase.Utc, (If((config.Mode And CInt(OutputMode.LocalTime)) = CInt(OutputMode.LocalTime), TimeBase.LocalTime, TimeBase.Tai)))

								  'OutputMode is an enum helping to split up the mode bit-mask
								  comboBoxMode.SelectedItem = If((config.Mode And CInt(OutputMode.IrigB)) = CInt(OutputMode.IrigB), OutputMode.IrigB, (If((config.Mode And CInt(OutputMode.Dcf77)) = CInt(OutputMode.Dcf77), OutputMode.Dcf77, (If((config.Mode And CInt(OutputMode.Ppx)) = CInt(OutputMode.Ppx), OutputMode.Ppx, OutputMode.Off)))))
								  checkBoxMuted.IsChecked = config.Mute

								  checkBoxModulation.IsChecked = (config.Mode And CInt(OutputMode.Modulated)) = CInt(OutputMode.Modulated)
								  'IrigExpressions is an enum helping to split up the IRIG-B type bit-mask
								  checkBoxControlFunctions.IsChecked = (config.IrigbConfiguration.Type And CInt(IrigExpressions.ControlFunctions)) = CInt(IrigExpressions.ControlFunctions)
								  checkBoxSBS.IsChecked = (config.IrigbConfiguration.Type And CInt(IrigExpressions.StraightBinarySeconds)) = CInt(IrigExpressions.StraightBinarySeconds)
								  checkBoxBcdYear.IsChecked = (config.IrigbConfiguration.Type And CInt(IrigExpressions.BcdYear)) = CInt(IrigExpressions.BcdYear)

								  'PpxMode is an enum helping to detect the PPX mode
								  comboBoxPpx.SelectedItem = CType(config.PpxConfiguration.Pps, PpxMode)
								  upDownPeriod.Value = config.PpxConfiguration.Ppm
								  upDownPulseWidth.Value = config.PpxConfiguration.PulseWidth
								  'TimeReference is an enum helping to detect the PPX reference
								  comboBoxTimeReference.SelectedItem = CType(config.PpxConfiguration.PulseSlope, TimeReference)

								  textBoxTriggerTime.Text = config.TriggerTime
							  End Sub)
		End Function

		''' <summary>Saves the general settings asynchronous.</summary>
		''' <returns></returns>
		Public Async Function SaveAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			Try
				Dim irigbSettings As IrigbConfiguration = New IrigbConfiguration()
				If checkBoxControlFunctions.IsChecked = True Then
					irigbSettings.Type = irigbSettings.Type Or CInt(IrigExpressions.ControlFunctions)
				End If
				If checkBoxSBS.IsChecked = True Then
					irigbSettings.Type = irigbSettings.Type Or CInt(IrigExpressions.StraightBinarySeconds)
				End If
				If checkBoxBcdYear.IsChecked = True Then
					irigbSettings.Type = irigbSettings.Type Or CInt(IrigExpressions.BcdYear)
				End If

				'must be supplied to satisfy the schema
				'must be supplied to satisfy the schema
				Dim newSettings As OutputConfiguration = New OutputConfiguration() With { _
					.Id = output, _
					.FriendlyName = [String].Empty, _
					.[Interface] = [String].Empty, _
					.Mode = CInt(comboBoxMode.SelectedItem), _
					.Mute = checkBoxMuted.IsChecked.Value = True, _
					.IrigbConfiguration = irigbSettings, _
					.PpxConfiguration = New PpxConfiguration() With { _
						.Pps = CInt(comboBoxPpx.SelectedItem), _
						.Ppm = upDownPeriod.Value.Value, _
						.PulseWidth = upDownPulseWidth.Value.Value, _
						.PulseSlope = CInt(comboBoxTimeReference.SelectedItem) _
					}, _
					.TriggerTime = textBoxTriggerTime.Text _
				}

				If CType(comboBoxTimeBase.SelectedItem, TimeBase) = TimeBase.Utc Then
					newSettings.Mode = newSettings.Mode Or CInt(OutputMode.Utc)
				ElseIf CType(comboBoxTimeBase.SelectedItem, TimeBase) = TimeBase.LocalTime Then
					newSettings.Mode = newSettings.Mode Or CInt(OutputMode.LocalTime)
				End If

				If checkBoxModulation.IsChecked = True Then
					newSettings.Mode = newSettings.Mode Or CInt(OutputMode.Modulated)
				End If

				'send the current values to the device
				Dim result As SetOutputConfigurationResponse = Await App.ApiClient.SetOutputConfigurationAsync(New OutputConfiguration() {newSettings})

				'check if the device saved the submitted values
				Dim savedSettings As OutputConfiguration = result.Body.SetOutputConfigurationResult.First(Function(c) c.Id = output)
				If savedSettings.Mode <> newSettings.Mode OrElse savedSettings.Mute <> newSettings.Mute OrElse savedSettings.IrigbConfiguration.Type <> newSettings.IrigbConfiguration.Type OrElse savedSettings.PpxConfiguration.Pps <> newSettings.PpxConfiguration.Pps OrElse savedSettings.PpxConfiguration.Ppm <> newSettings.PpxConfiguration.Ppm OrElse savedSettings.PpxConfiguration.PulseWidth <> newSettings.PpxConfiguration.PulseWidth OrElse savedSettings.PpxConfiguration.PulseSlope <> newSettings.PpxConfiguration.PulseSlope Then
					Xceed.Wpf.Toolkit.MessageBox.Show("Save failed: values not saved correctly!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
				End If
			Catch exp As Exception
				Xceed.Wpf.Toolkit.MessageBox.Show("Save failed:" & Environment.NewLine & exp.Message, "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
			End Try
		End Function
	End Class
End Namespace
