Imports FirstFloor.ModernUI.Windows
Imports OmicronLab.TicroControl.TicroService
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes

Namespace OmicronLab.TicroControl.Content.Settings.Ptp
	''' <summary>
	''' Interaction logic for General.xaml
	''' </summary>
	Partial Public Class General
		Inherits UserControl
		Implements IContent
		''' <summary>Gets the save command.</summary>
		''' <value>The save command.</value>
		Public Property SaveCommand() As AwaitableDelegateCommand
			Get
				Return m_SaveCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_SaveCommand = Value
			End Set
		End Property
		Private m_SaveCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="General"/> class.</summary>
		Public Sub New()
			SaveCommand = New AwaitableDelegateCommand(AddressOf SaveAsync)

			InitializeComponent()

			comboBoxProfile.ItemsSource = [Enum].GetValues(GetType(PtpMode)).Cast(Of PtpMode)()
			comboBoxMaxGMInaccuracy.ItemsSource = [Enum].GetValues(GetType(PtpAccuracy)).Cast(Of PtpAccuracy)()
			comboBoxIPv6Multicast.ItemsSource = [Enum].GetValues(GetType(IPv6MulticastType)).Cast(Of IPv6MulticastType)()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			Task.Run(Sub() UpdateValuesAsync().Wait())
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Dispatcher.Invoke(Function() Xceed.Wpf.Toolkit.MessageBox.Show("Please connect to a device to load the current values."))
				Return
			End If

			'Query the soap interface for the general settings - "Dim ptpSettings = App.ApiClient.GetPtpSettings()" executes the same synchronously
			Dim ptpSettings As GetPtpSettingsResponse = Await App.ApiClient.GetPtpSettingsAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the configuration values can be accessed at "ptpSettings.Body.GetPtpSettingsResult"
								  'when queried synchronous the configuration is returned directly
								  Dim settings As PtpConfiguration = ptpSettings.Body.GetPtpSettingsResult

								  comboBoxProfile.SelectedItem = settings.Mode
								  textBoxUserDescription.Text = settings.UserDescription
								  comboBoxMaxGMInaccuracy.SelectedItem = settings.MaxGrandmasterTimeInaccuracy
								  comboBoxIPv6Multicast.SelectedItem = settings.IPv6MulticastAddress
								  upDownDomainNumber.Value = settings.DomainNumber
							  End Sub)
		End Function

		''' <summary>Saves the general settings asynchronous.</summary>
		''' <returns></returns>
		Public Async Function SaveAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			Try
				'send the current values to the device
				Dim result As SetPtpGeneralSettingsResponse = Await App.ApiClient.SetPtpGeneralSettingsAsync(New GeneralPtpSettings() With { _
					.Mode = CType(comboBoxProfile.SelectedItem, PtpMode), _
					.UserDescription = textBoxUserDescription.Text, _
					.MaxGrandmasterTimeInaccuracy = CType(comboBoxMaxGMInaccuracy.SelectedItem, PtpAccuracy), _
					.IPv6MulticastAddress = CType(comboBoxIPv6Multicast.SelectedItem, IPv6MulticastType), _
					.DomainNumber = upDownDomainNumber.Value.Value _
				})

				'check if the device saved the submitted values
				Dim savedSettings As PtpConfiguration = result.Body.SetPtpGeneralSettingsResult
				If savedSettings.Mode <> CType(comboBoxProfile.SelectedItem, PtpMode) OrElse savedSettings.UserDescription <> textBoxUserDescription.Text OrElse savedSettings.MaxGrandmasterTimeInaccuracy <> CType(comboBoxMaxGMInaccuracy.SelectedItem, PtpAccuracy) OrElse savedSettings.IPv6MulticastAddress <> CType(comboBoxIPv6Multicast.SelectedItem, IPv6MulticastType) OrElse savedSettings.DomainNumber <> upDownDomainNumber.Value.Value Then
					Xceed.Wpf.Toolkit.MessageBox.Show("Save failed: values not saved correctly!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
				End If
			Catch exp As Exception
				Xceed.Wpf.Toolkit.MessageBox.Show("Save failed:" & Environment.NewLine & exp.Message, "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
			End Try
		End Function
	End Class
End Namespace
