Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes

Namespace OmicronLab.TicroControl.Content.Tools
	''' <summary>
	''' Interaction logic for DeviceControl.xaml
	''' </summary>
	Partial Public Class DeviceControl
		Inherits UserControl
		''' <summary>Gets the reboot command.</summary>
		''' <value>The reboot command.</value>
		Public Property RebootCommand() As AwaitableDelegateCommand
			Get
				Return m_RebootCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_RebootCommand = Value
			End Set
		End Property
		Private m_RebootCommand As AwaitableDelegateCommand
		''' <summary>Gets the reset command.</summary>
		''' <value>The reset command.</value>
		Public Property ResetCommand() As AwaitableDelegateCommand
			Get
				Return m_ResetCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_ResetCommand = Value
			End Set
		End Property
		Private m_ResetCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="DeviceControl"/> class.</summary>
		Public Sub New()
			RebootCommand = New AwaitableDelegateCommand(AddressOf RebootAsync)
			ResetCommand = New AwaitableDelegateCommand(AddressOf ResetAsync)

			InitializeComponent()
		End Sub

		''' <summary>Reboots the device asynchronous.</summary>
		''' <returns></returns>
		Public Async Function RebootAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			If Xceed.Wpf.Toolkit.MessageBox.Show("Really reboot the device? It will be offline for about 1 minute.", "Really reboot?", MessageBoxButton.YesNo, MessageBoxImage.Question) = MessageBoxResult.Yes Then
				Try
					'Attention: This action can fail, if the device reboots faster then SOAP can answer --> catch possible timeouts or errors
					Dim result As Boolean = Await App.ApiClient.RebootAsync()
					If result Then
						Xceed.Wpf.Toolkit.MessageBox.Show("Device is rebooting...", "Rebooting", MessageBoxButton.OK, MessageBoxImage.Information)
					Else
						Xceed.Wpf.Toolkit.MessageBox.Show("Device reboot failed!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
					End If
				Catch exp As Exception
					Xceed.Wpf.Toolkit.MessageBox.Show("Device reboot failed!" & Environment.NewLine & exp.Message, "Error", MessageBoxButton.OK, MessageBoxImage.[Error])

				End Try
			End If
		End Function

		''' <summary>Resets the device asynchronous.</summary>
		''' <returns></returns>
		Public Async Function ResetAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			If Xceed.Wpf.Toolkit.MessageBox.Show("Really reset the device? It will be offline for about 1 minute and you possibly need to change the end-point afterwards!", "Really reset?", MessageBoxButton.YesNo, MessageBoxImage.Question) = MessageBoxResult.Yes Then
				Try
					'Attention: This action can fail, if the device reboots faster then SOAP can answer --> catch possible timeouts or errors
					Dim result As Boolean = Await App.ApiClient.ResetAsync()
					If result Then
						Xceed.Wpf.Toolkit.MessageBox.Show("Device is resetting...", "Resetting", MessageBoxButton.OK, MessageBoxImage.Information)
					Else
						Xceed.Wpf.Toolkit.MessageBox.Show("Device reset failed!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
					End If
				Catch exp As Exception
					Xceed.Wpf.Toolkit.MessageBox.Show("Device reset failed!" & Environment.NewLine & exp.Message, "Error", MessageBoxButton.OK, MessageBoxImage.[Error])

				End Try
			End If
		End Function
	End Class
End Namespace
