Imports FirstFloor.ModernUI.Presentation
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows.Media

Namespace OmicronLab.TicroControl.Content
	''' <summary>
	''' A simple view model for configuring theme, font and accent colors.
	''' </summary>
	Public Class SettingsAppearanceViewModel
		Inherits NotifyPropertyChanged
		Private Const FontSmall As String = "small"
		Private Const FontLarge As String = "large"

		' 9 accent colors from metro design principles
		'private Color[] accentColors = new Color[]{
		'			Color.FromRgb(0x33, 0x99, 0xff),   // blue
		'			Color.FromRgb(0x00, 0xab, 0xa9),   // teal
		'			Color.FromRgb(0x33, 0x99, 0x33),   // green
		'			Color.FromRgb(0x8c, 0xbf, 0x26),   // lime
		'			Color.FromRgb(0xf0, 0x96, 0x09),   // orange
		'			Color.FromRgb(0xff, 0x45, 0x00),   // orange red
		'			Color.FromRgb(0xe5, 0x14, 0x00),   // red
		'			Color.FromRgb(0xff, 0x00, 0x97),   // magenta
		'			Color.FromRgb(0xa2, 0x00, 0xff),   // purple            
		'		};


		' 20 accent colors from Windows Phone 8
		' lime
		' green
		' emerald
		' teal
		' cyan
		' cobalt
		' indigo
		' violet
		' pink
		' magenta
		' crimson
		' red
		' orange
		' amber
		' yellow
		' brown
		' olive
		' steel
		' mauve
		' taupe
		Private m_accentColors As Color() = New Color() {Color.FromRgb(&HA4, &HC4, &H0), Color.FromRgb(&H60, &HA9, &H17), Color.FromRgb(&H0, &H8A, &H0), Color.FromRgb(&H0, &HAB, &HA9), Color.FromRgb(&H1B, &HA1, &HE2), Color.FromRgb(&H0, &H50, &HEF), _
			Color.FromRgb(&H6A, &H0, &HFF), Color.FromRgb(&HAA, &H0, &HFF), Color.FromRgb(&HF4, &H72, &HD0), Color.FromRgb(&HD8, &H0, &H73), Color.FromRgb(&HA2, &H0, &H25), Color.FromRgb(&HE5, &H14, &H0), _
			Color.FromRgb(&HFA, &H68, &H0), Color.FromRgb(&HF0, &HA3, &HA), Color.FromRgb(&HE3, &HC8, &H0), Color.FromRgb(&H82, &H5A, &H2C), Color.FromRgb(&H6D, &H87, &H64), Color.FromRgb(&H64, &H76, &H87), _
			Color.FromRgb(&H76, &H60, &H8A), Color.FromRgb(&H87, &H79, &H4E)}

		Private m_selectedAccentColor As Color
		Private m_themes As New LinkCollection()
		Private m_selectedTheme As Link
		Private m_selectedFontSize As String

		Public Sub New()
			' add the default themes
			Me.m_themes.Add(New Link() With { _
				.DisplayName = "dark", _
				.Source = AppearanceManager.DarkThemeSource _
			})
			Me.m_themes.Add(New Link() With { _
				.DisplayName = "light", _
				.Source = AppearanceManager.LightThemeSource _
			})

			Me.SelectedFontSize = If(AppearanceManager.Current.FontSize = FontSize.Large, FontLarge, FontSmall)
			SyncThemeAndColor()

			AddHandler AppearanceManager.Current.PropertyChanged, AddressOf OnAppearanceManagerPropertyChanged
		End Sub

		Private Sub SyncThemeAndColor()
			' synchronizes the selected viewmodel theme with the actual theme used by the appearance manager.
			Me.SelectedTheme = Me.m_themes.FirstOrDefault(Function(l) l.Source.Equals(AppearanceManager.Current.ThemeSource))

			' and make sure accent color is up-to-date
			Me.SelectedAccentColor = AppearanceManager.Current.AccentColor
		End Sub

		Private Sub OnAppearanceManagerPropertyChanged(sender As Object, e As PropertyChangedEventArgs)
			If e.PropertyName = "ThemeSource" OrElse e.PropertyName = "AccentColor" Then
				SyncThemeAndColor()
			End If
		End Sub

		Public ReadOnly Property Themes() As LinkCollection
			Get
				Return Me.m_themes
			End Get
		End Property

		Public ReadOnly Property FontSizes() As String()
			Get
				Return New String() {FontSmall, FontLarge}
			End Get
		End Property

		Public ReadOnly Property AccentColors() As Color()
			Get
				Return Me.m_accentColors
			End Get
		End Property

		Public Property SelectedTheme() As Link
			Get
				Return Me.m_selectedTheme
			End Get
			Set(value As Link)
				If Me.m_selectedTheme IsNot Value Then
					Me.m_selectedTheme = Value
					OnPropertyChanged("SelectedTheme")

					' and update the actual theme
					AppearanceManager.Current.ThemeSource = Value.Source
				End If
			End Set
		End Property

		Public Property SelectedFontSize() As String
			Get
				Return Me.m_selectedFontSize
			End Get
			Set(value As String)
				If Me.m_selectedFontSize <> Value Then
					Me.m_selectedFontSize = Value
					OnPropertyChanged("SelectedFontSize")

					AppearanceManager.Current.FontSize = If(Value = FontLarge, FontSize.Large, FontSize.Small)
				End If
			End Set
		End Property

		Public Property SelectedAccentColor() As Color
			Get
				Return Me.m_selectedAccentColor
			End Get
			Set(value As Color)
				If Me.m_selectedAccentColor <> Value Then
					Me.m_selectedAccentColor = Value
					OnPropertyChanged("SelectedAccentColor")

					AppearanceManager.Current.AccentColor = Value
				End If
			End Set
		End Property
	End Class
End Namespace
