Imports System.Collections.Generic
Imports System.Configuration
Imports System.Linq
Imports System.ServiceModel
Imports System.ServiceModel.Configuration
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports OmicronLab.TicroControl.TicroService
Imports System.Net
Imports System.Diagnostics

Namespace OmicronLab.TicroControl.Pages
	''' <summary>
	''' Interaction logic for Home.xaml
	''' </summary>
	Partial Public Class Home
		Inherits UserControl
		''' <summary>Gets the change endpoint command.</summary>
		''' <value>The change endpoint command.</value>
		Public Property ChangeEndpointCommand() As AwaitableDelegateCommand
			Get
				Return m_ChangeEndpointCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_ChangeEndpointCommand = Value
			End Set
		End Property
		Private m_ChangeEndpointCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="Home"/> class.</summary>
		Public Sub New()
			ChangeEndpointCommand = New AwaitableDelegateCommand(AddressOf ChangeEndpoint)

			InitializeComponent()

			'This is to be able to use invalid SSL certificates - can be removed when using valid ones
			ServicePointManager.ServerCertificateValidationCallback = Function(sender, certificate, chain, [error])
																				   Trace.WriteLine("WARNING: Certificate check skipped!")
																				   Return True
																			   End Function

			If [String].IsNullOrWhiteSpace(Properties.Settings.[Default].SoapEndpoint) Then
				Dim clientSection As ClientSection = TryCast(ConfigurationManager.GetSection("system.serviceModel/client"), ClientSection)
				Dim endpointCollection As ChannelEndpointElementCollection = TryCast(clientSection.ElementInformation.Properties([String].Empty).Value, ChannelEndpointElementCollection)
				Properties.Settings.[Default].SoapEndpoint = endpointCollection(0).Address.AbsoluteUri
				Properties.Settings.[Default].Save()
			End If

			textBoxEndpoint.Text = Properties.Settings.[Default].SoapEndpoint
			textBoxPassword.Text = Properties.Settings.[Default].Password

			Task.Run(Sub()
						 If Not App.Connect() Then
							 Dispatcher.Invoke(Sub()
												   labelStatus.Content = "Connection FAILED!"
												   Xceed.Wpf.Toolkit.MessageBox.Show(Properties.Resources.CantConnect, Properties.Resources.EndpointError, MessageBoxButton.OK, MessageBoxImage.[Error])
											   End Sub)
						 Else
							 Dispatcher.Invoke(Function() InlineAssignHelper(labelStatus.Content, "Connected!"))
						 End If
					 End Sub)
		End Sub

		''' <summary>Changes the endpoint.</summary>
		Private Async Function ChangeEndpoint() As Task
			Try
				'Please see App.Connect for a synchronous approach
				Dim endpoint As String = textBoxEndpoint.Text
				Dim client As ApiClient = Await Task(Of ApiClient).Run(Function() App.GetApiClient(endpoint))

				Dim doLogin As Boolean = False
				Dim status As String = [String].Empty
				Try
					status = (Await client.StatusAsync()).Body.StatusResult
				Catch fault As FaultException
					'If login is enabled an "Authentication" fault is returned
					If fault.Code.Name = "Authentication" Then
						doLogin = True
					Else
						Throw
					End If
				Catch exp As AggregateException
					'If login is enabled an "Authentication" fault is returned
					If TypeOf exp.InnerException Is FaultException AndAlso TryCast(exp.InnerException, FaultException).Code.Name = "Authentication" Then
						doLogin = True
					Else
						Throw
					End If
				End Try
				If doLogin Then
					If (Await client.LoginAsync(textBoxPassword.Text)).Body.LoginResult Then
						status = (Await client.StatusAsync()).Body.StatusResult
					Else
						Xceed.Wpf.Toolkit.MessageBox.Show(Properties.Resources.WrongPassword, [String].Empty, MessageBoxButton.OK, MessageBoxImage.[Error])
						Return
					End If
				End If

				If status = "OK" Then
					labelStatus.Content = "Connected!"

					Properties.Settings.[Default].SoapEndpoint = textBoxEndpoint.Text
					Properties.Settings.[Default].Password = textBoxPassword.Text
					Properties.Settings.[Default].Save()
					App.ApiClient = client

					Xceed.Wpf.Toolkit.MessageBox.Show(Properties.Resources.EndpointSaved, [String].Empty, MessageBoxButton.OK, MessageBoxImage.Information)
				Else
					Xceed.Wpf.Toolkit.MessageBox.Show(Properties.Resources.ServiceNotOk, [String].Empty, MessageBoxButton.OK, MessageBoxImage.Warning)
				End If
			Catch exp As Exception
				Xceed.Wpf.Toolkit.MessageBox.Show(Properties.Resources.EndpointError & Environment.NewLine & exp.ToString(), Properties.Resources.EndpointError, MessageBoxButton.OK, MessageBoxImage.[Error])
			End Try
		End Function
		Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
			target = value
			Return value
		End Function
	End Class
End Namespace
