Imports FirstFloor.ModernUI.Windows
Imports OmicronLab.TicroControl.TicroService
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes

Namespace OmicronLab.TicroControl.Pages.Settings
	''' <summary>
	''' Interaction logic for General.xaml
	''' </summary>
	Public Partial Class General
		Inherits UserControl
		Implements IContent
		''' <summary>Gets the save command.</summary>
		''' <value>The save command.</value>
		Public Property SaveCommand() As AwaitableDelegateCommand
			Get
				Return m_SaveCommand
			End Get
			Private Set
				m_SaveCommand = Value
			End Set
		End Property
		Private m_SaveCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="General"/> class.</summary>
		Public Sub New()
			SaveCommand = New AwaitableDelegateCommand(AddressOf SaveAsync)

			InitializeComponent()

			comboBoxOutputsActive.ItemsSource = [Enum].GetValues(GetType(OutputActivationMode)).Cast(Of OutputActivationMode)()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			Task.Run(Sub() UpdateValuesAsync().Wait())
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Dispatcher.Invoke(Function() Xceed.Wpf.Toolkit.MessageBox.Show("Please connect to a device to load the current values and available time zones."))
				Return
			End If

			'Query the soap interface for the available time zones - "Dim result = App.ApiClient.GetAvailableTimezones()" executes the same synchronously
			Dim result As GetAvailableTimezonesResponse = Await App.ApiClient.GetAvailableTimezonesAsync()

			'Query the soap interface for the general settings - "Dim generalSettings = App.ApiClient.GetGeneralSettings()" executes the same synchronously
			Dim generalSettings As GetGeneralSettingsResponse = Await App.ApiClient.GetGeneralSettingsAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the array can be accessed at "result.Body.GetAvailableTimezonesResult"
								  'when queried synchronous the array is returned directly
								  comboBoxTimezone.ItemsSource = result.Body.GetAvailableTimezonesResult

								  'if queried asynchronous the configuration values can be accessed at "generalSettings.Body.GetGeneralSettingsResult"
								  'when queried synchronous the configuration is returned directly
								  Dim settings As GeneralConfiguration = generalSettings.Body.GetGeneralSettingsResult

								  comboBoxTimezone.SelectedValue = settings.Timezone
								  upDownLeapSeconds.Value = settings.LeapSeconds
								  comboBoxOutputsActive.SelectedItem = settings.OutputsActive
								  textBoxDstDateTime.Text = settings.DstDateTime
								  upDownDstOffset.Value = settings.DstOffset
								  checkBoxDstFlag.IsChecked = If(settings.DstFlag = 1, True, False)
							  End Sub)
		End Function

		''' <summary>Saves the general settings asynchronous.</summary>
		''' <returns></returns>
		Public Async Function SaveAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			Try
				'send the current values to the device
				Dim result As SetGeneralSettingsResponse = Await App.ApiClient.SetGeneralSettingsAsync(New GeneralConfiguration() With { _
					.Timezone = comboBoxTimezone.SelectedValue.ToString(), _
					.LeapSeconds = upDownLeapSeconds.Value.Value, _
					.OutputsActive = CType(comboBoxOutputsActive.SelectedItem, OutputActivationMode), _
					.DstDateTime = textBoxDstDateTime.Text, _
					.DstOffset = upDownDstOffset.Value.Value, _
					.DstFlag = If(checkBoxDstFlag.IsChecked = True, 1, 0) _
				})

				'check if the device saved the submitted values
				Dim savedSettings As GeneralConfiguration = result.Body.SetGeneralSettingsResult
				If savedSettings.Timezone <> comboBoxTimezone.SelectedValue.ToString() OrElse savedSettings.LeapSeconds <> upDownLeapSeconds.Value.Value OrElse savedSettings.OutputsActive <> CType(comboBoxOutputsActive.SelectedItem, OutputActivationMode) OrElse savedSettings.DstDateTime <> textBoxDstDateTime.Text OrElse savedSettings.DstOffset <> upDownDstOffset.Value.Value Then
					Xceed.Wpf.Toolkit.MessageBox.Show("Save failed: values not saved correctly!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
				End If
			Catch exp As Exception
				Xceed.Wpf.Toolkit.MessageBox.Show("Save failed:" & Environment.NewLine & exp.Message, "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
			End Try
		End Function
	End Class
End Namespace
