Imports FirstFloor.ModernUI.Windows
Imports OmicronLab.TicroControl.TicroService
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes

Namespace OmicronLab.TicroControl.Pages.Settings
	''' <summary>
	''' Interaction logic for Log.xaml
	''' </summary>
	Partial Public Class Log
		Inherits UserControl
		Implements IContent
		''' <summary>Gets the save command.</summary>
		''' <value>The save command.</value>
		Public Property SaveCommand() As AwaitableDelegateCommand
			Get
				Return m_SaveCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_SaveCommand = Value
			End Set
		End Property
		Private m_SaveCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="General"/> class.</summary>
		Public Sub New()
			SaveCommand = New AwaitableDelegateCommand(AddressOf SaveAsync)

			InitializeComponent()

			comboBoxLogLevel.ItemsSource = [Enum].GetValues(GetType(LogLevel)).Cast(Of LogLevel)()
			comboBoxLogTarget.ItemsSource = [Enum].GetValues(GetType(LogTarget)).Cast(Of LogTarget)()
			comboBoxRemoteTransport.ItemsSource = [Enum].GetValues(GetType(LogTransport)).Cast(Of LogTransport)()

			comboBoxKernelSource.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxPtpSource.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxClockStateSource.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()

			comboBoxNotifications.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()

			comboBoxPtpDisabledNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxPtpFaultyNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxPtpListeningNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxPtpSlaveNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxPtpUncalibratedNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()

			comboBoxClockHoldoverNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxClockInitNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxClockLockedNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxClockLockingNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxClockUnlockedNotification.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			Task.Run(Sub() UpdateValuesAsync().Wait())
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Dispatcher.Invoke(Function() Xceed.Wpf.Toolkit.MessageBox.Show("Please connect to a device to load the current values."))
				Return
			End If

			'Query the soap interface for the syslog settings - "Dim logSettings = App.ApiClient.GetSyslogSettings()" executes the same synchronously
			Dim logSettings As GetSyslogSettingsResponse = Await App.ApiClient.GetSyslogSettingsAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the configuration values can be accessed at "logSettings.Body.GetSyslogSettingsResult"
								  'when queried synchronous the configuration is returned directly
								  Dim settings As SyslogConfiguration = logSettings.Body.GetSyslogSettingsResult

								  comboBoxLogLevel.SelectedItem = settings.LogLevel
								  comboBoxLogTarget.SelectedItem = settings.LogTarget
								  textBoxRemoteHost.Text = settings.RemoteHost
								  upDownRemotePort.Value = settings.RemotePort
								  comboBoxRemoteTransport.SelectedItem = settings.RemoteTransport
								  comboBoxKernelSource.SelectedItem = settings.KernelSource
								  comboBoxPtpSource.SelectedItem = settings.PtpDooSource
								  comboBoxClockStateSource.SelectedItem = settings.DatasyncSource

								  comboBoxNotifications.SelectedItem = settings.Notifications
								  textBoxSmtpHost.Text = settings.SmtpHost
								  upDownSmtpPort.Value = settings.SmtpPort
								  textBoxSender.Text = settings.SmtpFrom
								  textBoxRecipient.Text = settings.MailTo
								  comboBoxPtpDisabledNotification.SelectedItem = settings.PtpDisabledNotification
								  comboBoxPtpFaultyNotification.SelectedItem = settings.PtpFaultyNotification
								  comboBoxPtpListeningNotification.SelectedItem = settings.PtpListeningNotification
								  comboBoxPtpSlaveNotification.SelectedItem = settings.PtpSlaveNotification
								  comboBoxPtpUncalibratedNotification.SelectedItem = settings.PtpUncalibratedNotification
								  comboBoxClockHoldoverNotification.SelectedItem = settings.ClockHoldoverNotification
								  comboBoxClockInitNotification.SelectedItem = settings.ClockInitNotification
								  comboBoxClockLockedNotification.SelectedItem = settings.ClockLockNotification
								  comboBoxClockLockingNotification.SelectedItem = settings.ClockLockingNotification
								  comboBoxClockUnlockedNotification.SelectedItem = settings.ClockUnlockNotification
							  End Sub)
		End Function

		''' <summary>Saves the general settings asynchronous.</summary>
		''' <returns></returns>
		Public Async Function SaveAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			Try
				'send the current values to the device

				Dim result As SetSyslogSettingsResponse = Await App.ApiClient.SetSyslogSettingsAsync(New SyslogConfiguration() With { _
					.LogLevel = CType(comboBoxLogLevel.SelectedItem, LogLevel), _
					.LogTarget = CType(comboBoxLogTarget.SelectedItem, LogTarget), _
					.RemoteHost = textBoxRemoteHost.Text, _
					.RemotePort = upDownRemotePort.Value.Value, _
					.RemoteTransport = CType(comboBoxRemoteTransport.SelectedItem, LogTransport), _
					.KernelSource = CType(comboBoxKernelSource.SelectedItem, PropertyState), _
					.PtpDooSource = CType(comboBoxPtpSource.SelectedItem, PropertyState), _
					.DatasyncSource = CType(comboBoxClockStateSource.SelectedItem, PropertyState), _
					.Notifications = CType(comboBoxNotifications.SelectedItem, PropertyState), _
					.SmtpHost = textBoxSmtpHost.Text, _
					.SmtpPort = upDownSmtpPort.Value.Value, _
					.SmtpFrom = textBoxSender.Text, _
					.MailTo = textBoxRecipient.Text, _
					.PtpDisabledNotification = CType(comboBoxPtpDisabledNotification.SelectedItem, PropertyState), _
					.PtpFaultyNotification = CType(comboBoxPtpFaultyNotification.SelectedItem, PropertyState), _
					.PtpListeningNotification = CType(comboBoxPtpListeningNotification.SelectedItem, PropertyState), _
					.PtpSlaveNotification = CType(comboBoxPtpSlaveNotification.SelectedItem, PropertyState), _
					.PtpUncalibratedNotification = CType(comboBoxPtpUncalibratedNotification.SelectedItem, PropertyState), _
					.ClockHoldoverNotification = CType(comboBoxClockHoldoverNotification.SelectedItem, PropertyState), _
					.ClockInitNotification = CType(comboBoxClockInitNotification.SelectedItem, PropertyState), _
					.ClockLockNotification = CType(comboBoxClockLockedNotification.SelectedItem, PropertyState), _
					.ClockLockingNotification = CType(comboBoxClockLockingNotification.SelectedItem, PropertyState), _
					.ClockUnlockNotification = CType(comboBoxClockUnlockedNotification.SelectedItem, PropertyState) _
				})

				'check if the device saved the submitted values
				Dim savedSettings As SyslogConfiguration = result.Body.SetSyslogSettingsResult
				If savedSettings.LogLevel <> CType(comboBoxLogLevel.SelectedItem, LogLevel) OrElse savedSettings.LogTarget <> CType(comboBoxLogTarget.SelectedItem, LogTarget) OrElse savedSettings.RemoteHost <> textBoxRemoteHost.Text OrElse savedSettings.RemotePort <> upDownRemotePort.Value.Value OrElse savedSettings.RemoteTransport <> CType(comboBoxRemoteTransport.SelectedItem, LogTransport) OrElse savedSettings.KernelSource <> CType(comboBoxKernelSource.SelectedItem, PropertyState) OrElse savedSettings.PtpDooSource <> CType(comboBoxPtpSource.SelectedItem, PropertyState) OrElse savedSettings.DatasyncSource <> CType(comboBoxClockStateSource.SelectedItem, PropertyState) OrElse savedSettings.Notifications <> CType(comboBoxNotifications.SelectedItem, PropertyState) OrElse savedSettings.SmtpHost <> textBoxSmtpHost.Text OrElse savedSettings.SmtpPort <> upDownSmtpPort.Value.Value OrElse savedSettings.SmtpFrom <> textBoxSender.Text OrElse savedSettings.MailTo <> textBoxRecipient.Text OrElse savedSettings.PtpDisabledNotification <> CType(comboBoxPtpDisabledNotification.SelectedItem, PropertyState) OrElse savedSettings.PtpFaultyNotification <> CType(comboBoxPtpFaultyNotification.SelectedItem, PropertyState) OrElse savedSettings.PtpListeningNotification <> CType(comboBoxPtpListeningNotification.SelectedItem, PropertyState) OrElse savedSettings.PtpSlaveNotification <> CType(comboBoxPtpSlaveNotification.SelectedItem, PropertyState) OrElse savedSettings.PtpUncalibratedNotification <> CType(comboBoxPtpUncalibratedNotification.SelectedItem, PropertyState) OrElse savedSettings.ClockHoldoverNotification <> CType(comboBoxClockHoldoverNotification.SelectedItem, PropertyState) OrElse savedSettings.ClockInitNotification <> CType(comboBoxClockInitNotification.SelectedItem, PropertyState) OrElse savedSettings.ClockLockNotification <> CType(comboBoxClockLockedNotification.SelectedItem, PropertyState) OrElse savedSettings.ClockLockingNotification <> CType(comboBoxClockLockingNotification.SelectedItem, PropertyState) OrElse savedSettings.ClockUnlockNotification <> CType(comboBoxClockUnlockedNotification.SelectedItem, PropertyState) Then
					Xceed.Wpf.Toolkit.MessageBox.Show("Save failed: values not saved correctly!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
				End If
			Catch exp As Exception
				Xceed.Wpf.Toolkit.MessageBox.Show("Save failed:" & Environment.NewLine & exp.Message, "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
			End Try
		End Function
	End Class
End Namespace
