Imports FirstFloor.ModernUI.Windows
Imports OmicronLab.TicroControl.Helper
Imports OmicronLab.TicroControl.TicroService
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes

Namespace OmicronLab.TicroControl.Pages.Settings
	''' <summary>
	''' Interaction logic for Security.xaml
	''' </summary>
	Partial Public Class Security
		Inherits UserControl
		Implements IContent
		''' <summary>Gets the save command.</summary>
		''' <value>The save command.</value>
		Public Property SaveCommand() As AwaitableDelegateCommand
			Get
				Return m_SaveCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_SaveCommand = value
			End Set
		End Property
		Private m_SaveCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="General"/> class.</summary>
		Public Sub New()
			SaveCommand = New AwaitableDelegateCommand(AddressOf SaveAsync)

			InitializeComponent()

			comboBoxAccess.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxProtocol.ItemsSource = [Enum].GetValues(GetType(HttpMode)).Cast(Of HttpMode)()
			comboBoxServices.ItemsSource = [Enum].GetValues(GetType(ServiceMode)).Cast(Of ServiceMode)()

			comboBoxSshLogin.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxPtpDoo.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxOmFind.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxZeroconf.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxFtp.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
			comboBoxTFtp.ItemsSource = [Enum].GetValues(GetType(PropertyState)).Cast(Of PropertyState)()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			Task.Run(Sub() UpdateValuesAsync().Wait())
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
		End Sub

		''' <summary>Updates the values.</summary>
		''' <returns></returns>
		Private Async Function UpdateValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Dispatcher.Invoke(Function() Xceed.Wpf.Toolkit.MessageBox.Show("Please connect to a device to load the current values."))
				Return
			End If

			'Query the soap interface for the system settings - "Dim systemSettings = App.ApiClient.GetSystemSettings()" executes the same synchronously
			Dim systemSettings As GetSystemSettingsResponse = Await App.ApiClient.GetSystemSettingsAsync()

			'Query the soap interface for the HTTP settings - "Dim httpSettings = App.ApiClient.GetHttpSettings()" executes the same synchronously
			Dim httpSettings As GetHttpSettingsResponse = Await App.ApiClient.GetHttpSettingsAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the configuration values can be accessed at "systemSettings.Body.GetSystemSettingsResult"
								  'when queried synchronous the configuration is returned directly
								  Dim settings As SystemConfiguration = systemSettings.Body.GetSystemSettingsResult

								  'if queried asynchronous the configuration values can be accessed at "httpSettings.Body.GetHttpSettingsResult"
								  'when queried synchronous the configuration is returned directly
								  Dim webSettings As HttpConfiguration = httpSettings.Body.GetHttpSettingsResult

								  comboBoxAccess.SelectedItem = webSettings.Authentication
								  comboBoxProtocol.SelectedItem = webSettings.Mode
								  'ServiceMode is a helper enum to parse the service access mode
								  comboBoxServices.SelectedItem = CType(webSettings.ServiceAccess, ServiceMode)

								  comboBoxSshLogin.SelectedItem = settings.Login
								  comboBoxPtpDoo.SelectedItem = settings.PtpDoo
								  comboBoxOmFind.SelectedItem = settings.OMFlite
								  comboBoxZeroconf.SelectedItem = settings.Avahi
								  comboBoxFtp.SelectedItem = settings.VSFTPd
								  comboBoxTFtp.SelectedItem = settings.TFTPd
							  End Sub)
		End Function

		''' <summary>Saves the general settings asynchronous.</summary>
		''' <returns></returns>
		Public Async Function SaveAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			Try
				'send the current system settings values to the device
				Dim systemResult As SetSystemSettingsResponse = Await App.ApiClient.SetSystemSettingsAsync(New SystemConfiguration() With { _
					.Password = textBoxPassword.Text, _
					.Login = CType(comboBoxSshLogin.SelectedItem, PropertyState), _
					.PtpDoo = CType(comboBoxPtpDoo.SelectedItem, PropertyState), _
					.OMFlite = CType(comboBoxOmFind.SelectedItem, PropertyState), _
					.Avahi = CType(comboBoxZeroconf.SelectedItem, PropertyState), _
					.VSFTPd = CType(comboBoxFtp.SelectedItem, PropertyState), _
					.TFTPd = CType(comboBoxTFtp.SelectedItem, PropertyState) _
				})

				'check if the device saved the submitted values
				Dim savedSystemSettings As SystemConfiguration = systemResult.Body.SetSystemSettingsResult
				'The password can't be checked directly, as it is hashed right away
				If savedSystemSettings.Login <> CType(comboBoxSshLogin.SelectedItem, PropertyState) OrElse savedSystemSettings.PtpDoo <> CType(comboBoxPtpDoo.SelectedItem, PropertyState) OrElse savedSystemSettings.OMFlite <> CType(comboBoxOmFind.SelectedItem, PropertyState) OrElse savedSystemSettings.Avahi <> CType(comboBoxZeroconf.SelectedItem, PropertyState) OrElse savedSystemSettings.VSFTPd <> CType(comboBoxFtp.SelectedItem, PropertyState) OrElse savedSystemSettings.TFTPd <> CType(comboBoxTFtp.SelectedItem, PropertyState) Then
					Xceed.Wpf.Toolkit.MessageBox.Show("Save system settings failed: values not saved correctly!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
				End If

				'send the current web settings values to the device
				'Attention: This action can fail, if the device reboots faster then SOAP can answer --> catch possible timeouts or errors
				Dim result As SetHttpSettingsResponse = Await App.ApiClient.SetHttpSettingsAsync(New HttpConfiguration() With { _
					.Mode = CType(comboBoxProtocol.SelectedItem, HttpMode), _
					.Authentication = CType(comboBoxAccess.SelectedItem, PropertyState), _
					.ServiceAccess = CInt(comboBoxServices.SelectedItem) _
				})

				'check if the device saved the submitted values
				Dim savedSettings As HttpConfiguration = result.Body.SetHttpSettingsResult
				If savedSettings.Mode <> CType(comboBoxProtocol.SelectedItem, HttpMode) OrElse savedSettings.Authentication <> CType(comboBoxAccess.SelectedItem, PropertyState) OrElse savedSettings.ServiceAccess <> CInt(comboBoxServices.SelectedItem) Then
					Xceed.Wpf.Toolkit.MessageBox.Show("Save web settings failed: values not saved correctly!", "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
				End If
			Catch exp As Exception
				Xceed.Wpf.Toolkit.MessageBox.Show("Save failed:" & Environment.NewLine & exp.Message, "Error", MessageBoxButton.OK, MessageBoxImage.[Error])
			End Try
		End Function
	End Class
End Namespace
