Imports FirstFloor.ModernUI.Windows
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports OmicronLab.TicroControl.TicroService

Namespace OmicronLab.TicroControl.Pages.Status
	''' <summary>
	''' Interaction logic for Network.xaml
	''' </summary>
	Partial Public Class Network
		Inherits UserControl
		Implements IContent
		''' <summary>Gets the refresh command.</summary>
		''' <value>The refresh command.</value>
		Public Property RefreshCommand() As AwaitableDelegateCommand
			Get
				Return m_RefreshCommand
			End Get
			Private Set(value As AwaitableDelegateCommand)
				m_RefreshCommand = value
			End Set
		End Property
		Private m_RefreshCommand As AwaitableDelegateCommand

		''' <summary>Initializes a new instance of the <see cref="Network"/> class.</summary>
		Public Sub New()
			RefreshCommand = New AwaitableDelegateCommand(AddressOf RefreshValuesAsync)

			InitializeComponent()
		End Sub

		''' <summary>Called when navigation to a content fragment begins.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		Public Sub OnFragmentNavigation(e As FirstFloor.ModernUI.Windows.Navigation.FragmentNavigationEventArgs) Implements IContent.OnFragmentNavigation
		End Sub

		''' <summary>Called when this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedFrom
		End Sub

		''' <summary>Called when a this instance becomes the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <exception cref="System.NotImplementedException"></exception>
		Public Sub OnNavigatedTo(e As FirstFloor.ModernUI.Windows.Navigation.NavigationEventArgs) Implements IContent.OnNavigatedTo
			Task.Run(Sub() RefreshCommand.Execute(Nothing))
		End Sub

		''' <summary>Called just before this instance is no longer the active content in a frame.</summary>
		''' <param name="e">An object that contains the navigation data.</param>
		''' <remarks>The method is also invoked when parent frames are about to navigate.</remarks>
		Public Sub OnNavigatingFrom(e As FirstFloor.ModernUI.Windows.Navigation.NavigatingCancelEventArgs) Implements IContent.OnNavigatingFrom
		End Sub

		''' <summary>Refreshes the values asynchronous.</summary>
		''' <returns></returns>
		Private Async Function RefreshValuesAsync() As Task
			If App.ApiClient Is Nothing Then
				Return
			End If

			'Query the soap interface for the network settings - "Dim networkSettings = App.ApiClient.GetNetworkSettings()" executes the same synchronously
			Dim networkSettings As GetNetworkSettingsResponse = Await App.ApiClient.GetNetworkSettingsAsync()

			Dispatcher.Invoke(Sub()
								  'if queried asynchronous the configuration values can be accessed at "networkSettings.Body.GetNetworkSettingsResult"
								  'when queried synchronous the configuration is returned directly
								  Dim settings As NetworkConfiguration = networkSettings.Body.GetNetworkSettingsResult

								  labelHostName.Content = settings.Hostname
								  labelDomainName.Content = settings.Domain
								  labelNetworkPort.Content = settings.DefaultPort

								  labelIPv4Configuration.Content = settings.IPv4Configuration.Mode
								  labelIPv4Address.Content = settings.IPv4Configuration.Address
								  labelIPv4Mask.Content = settings.IPv4Configuration.Netmask
								  labelIPv4Gateway.Content = settings.IPv4Configuration.Gateway
								  labelIPv4Nameserver1.Content = settings.IPv4Configuration.Nameserver1
								  labelIPv4Nameserver2.Content = settings.IPv4Configuration.Nameserver2

								  labelIPv6Configuration.Content = settings.IPv6Configuration.Mode
								  labelIPv6Address.Content = settings.IPv6Configuration.Address
								  labelIPv6Mask.Content = settings.IPv6Configuration.Netmask
								  labelIPv6Gateway.Content = settings.IPv6Configuration.Gateway
								  labelIPv6Nameserver1.Content = settings.IPv6Configuration.Nameserver1
								  labelIPv6Nameserver2.Content = settings.IPv6Configuration.Nameserver2
							  End Sub)
		End Function
	End Class
End Namespace
